"""This module is for the translation mapping data of the
following document:

Document file: int_and_number_arithmetic_operations.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Int and Number basic arithmetic operations':
    '',

    'This page explains basic arithmetic operations of the `Int` and `Number` classes, like addition, multiplication, or incremental addition.':  # noqa
    '',

    '## Common behaviors':
    '',

    'You can calculate the `Int` and `Number` values with the Python built-in values, such as `int` or `float`\\, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```':  # noqa
    '',

    'Also, arithmetic operations with the same types (e.g., `Int` and `Int`) are supported:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```':  # noqa
    '',

    'Arithmetic operations are not supported if the left value is the Python built-in value. For instance, the following code raises an exception:':  # noqa
    '',

    '```py\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\n\n# This will raise the error!\nint_1 = 20 + int_1\n```':  # noqa
    '',

    '':
    '',

    '```\nTypeError: unsupported operand type(s) for +: \'int\' and \'Int\'\n```':  # noqa
    '',

    '':
    '',

    '## Addition':
    '',

    'You can add values with the `+` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 + 20\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 + int_2\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10) + ap.Int(20)\nassert int_1 == 30\n```':  # noqa
    '',

    'Also, you can use the `+=` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 += 20\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(20)\nint_1 += int_2\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '## Subtraction':
    '',

    'You can subtract values with the `-` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_1 = int_1 - 10\nassert int_1 == 20\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(30)\nint_2: ap.Int = ap.Int(20)\nint_1 = int_1 - int_2\nassert int_1 == 10\n```':  # noqa
    '',

    'Also, you can use the `-=` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(50)\nint_1 -= 30\nassert int_1 == 20\n```':  # noqa
    '',

    '':
    '',

    '## Multiplication':
    '',

    'You can multiply values with the `*` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 * 3\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(5)\nint_1 = int_1 * int_2\nassert int_1 == 50\n```':  # noqa
    '',

    'Also, you can use the `*=` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 *= 3\nassert int_1 == 30\n```':  # noqa
    '',

    '':
    '',

    '## Division':
    '',

    'You can divide values with the `/` operator. A return value becomes a `Number` value, not an `Int`\\.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 / 4\nassert number_1 == 2.5\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: ap.Int = ap.Int(4)\nnumber_1: ap.Number = int_1 / int_2\nassert number_1 == 2.5\n```':  # noqa
    '',

    'Also, you can use the `/=` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10)\nnumber_1 /= 4\nassert number_1 == 2.5\n```':  # noqa
    '',

    '':
    '',

    '## Floor division':
    '',

    'You can divide and floor values with the `//` operator. A return value becomes an `Int` value, not a `Number`\\.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_1 = int_1 // 4\nassert int_1 == 2\n```':  # noqa
    '',

    '':
    '',

    '## Modulo':
    '',

    'You can use the modulo operation with the `%` operator.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nint_2: int = int_1 % 3\nassert int_2 == 1\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nnumber_1: ap.Number = ap.Number(10.5)\nnumber_2: ap.Number = number_1 % 3\nassert number_2 == 1.5\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_slice.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class slice interface':
    '',

    'This page explains the `Array` class `slice` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `slice` method interface extracts the specified index range array\'s values and returns a new array.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `slice` method requires the `start` and `end` arguments (`int` or `Int` values) and returns a new array.\n\nIf you specify 1 to the `start` argument and 3 to the `end` argument, this method behaves like the Python built-in list slice of `[1:3]`.\n\nAn original array is not modified.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3, 4])\nsliced_arr: ap.Array[int] = arr.slice(start=1, end=3)\nassert sliced_arr == [2, 3]\nassert arr == [1, 2, 3, 4]\n```':  # noqa
    '',

    '':
    '',

    '## slice API':
    '',

    '<!-- Docstring: apysc._type.array.Array.slice -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `slice(self, *, start:Union[int, apysc._type.int.Int, NoneType]=None, end:Union[int, apysc._type.int.Int, NoneType]=None) -> \'Array\'`<hr>\n\n**[Interface summary]** Slice this array by specified start and end indexes.<hr>\n\n**[Parameters]**\n\n- `start`: Int or int or None, default None\n  - Slicing start index.\n- `end`: Int or int or None, default None\n  - Slicing end index (a result array does not contain this index).\n\n<hr>\n\n**[Returns]**\n\n- `sliced_arr`: Array\n  - Sliced array.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3, 4])\n>>> arr.slice(start=1, end=3)\nArray([2, 3])\n\n>>> arr.slice(start=1)\nArray([2, 3, 4])\n\n>>> arr.slice(end=2)\nArray([1, 2])\n```':  # noqa
    '',

}



"""Class implementation for the inheritance of each mouse
event interface.
"""

from apysc._event.click_interface import ClickInterface
from apysc._event.double_click_interface import DoubleClickInterface
from apysc._event.mouse_down_interface import MouseDownInterface
from apysc._event.mouse_move_interface import MouseMoveInterface
from apysc._event.mouse_out_interface import MouseOutInterface
from apysc._event.mouse_over_interface import MouseOverInterface
from apysc._event.mouse_up_interface import MouseUpInterface


class MouseEventInterfaces(
        ClickInterface, DoubleClickInterface, MouseDownInterface,
        MouseUpInterface, MouseOverInterface, MouseOutInterface,
        MouseMoveInterface):
    """Class implementation for the inheritance of each mouse
    event interface.
    """



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_scale_from_point.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# GraphicsBase get_scale_from_point and set_scale_from_point interfaces':  # noqa
    '',

    'This page explains the `GraphicsBase` class (base class of each graphics, such as the `Rectangle`) `get_scale_x_from_point`, `get_scale_y_from_point`, `set_scale_x_from_point`, and `set_scale_y_from_point` method interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `set_scale_x_from_point` method changes the object\'s horizontal scale from a given x-coordinate. Similarly, the `set_scale_y_from_point` method changes the object\'s vertical scale from a given y-coordinate.\n\nThe `scale_x_from_center` and `scale_y_from_center` interfaces are property, but the `set_scale_x_from_point` and `set_scale_y_from_point` interfaces are methods since these interfaces require a coodinate argument.\n\nSimilarly, the `get_scale_x_from_point` and `get_scale_y_from_point` methods will return the current scale from a given point. These interfaces also require a coordinate argument.\n\nReturn value is set for each coordinate. For example, if you set the scale-x value at the 50px x-coordinate, 100px x-coordinate scale will not be affected.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `get_scale_x_from_point` method requires the `x` argument (`Int` value), and the `set_scale_x_from_point` requires the `scale_x` (`Number` value) and `x` arguments.\n\nThe following example creates three rectangles and increments (or decrements) for each rectangle scale-x value. The top rectangle scales from the left-x position. The middle one scales from the center-x. And the bottom one scales from the right-x.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _Options(TypedDict):\n    rectangle: ap.Rectangle\n    x: ap.Int\n    direction: ap.Int\n\n\ndef on_timer(e: ap.TimerEvent, options: _Options) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    x: ap.Int = options[\'x\']\n    direction: ap.Int = options[\'direction\']\n    current_scale_x: ap.Number = rectangle.get_scale_x_from_point(x=x)\n    current_scale_x += direction * 0.03\n    rectangle.set_scale_x_from_point(scale_x=current_scale_x, x=x)\n    with ap.If(current_scale_x >= 2.0):\n        direction *= -1\n    with ap.If(current_scale_x <= 0.0):\n        direction *= -1\n\n\nap.Stage(\n    stage_width=150, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\ntop_rect: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nmiddle_rect: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nbottom_rect: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\ntop_rect_direction: ap.Int = ap.Int(1)\noptions: _Options = {\n    \'rectangle\': top_rect, \'x\': ap.Int(50),\n    \'direction\': top_rect_direction}\ntop_rect_timer: ap.Timer = ap.Timer(\n    on_timer, delay=ap.FPS.FPS_60,\n    options=options)\ntop_rect_timer.start()\n\nmiddle_rect_direction: ap.Int = ap.Int(1)\noptions = {\n    \'rectangle\': middle_rect, \'x\': ap.Int(75),\n    \'direction\': middle_rect_direction}\nmiddle_rect_timer: ap.Timer = ap.Timer(\n    on_timer, delay=ap.FPS.FPS_60,\n    options=options)\nmiddle_rect_timer.start()\n\nbottom_rect_direction: ap.Int = ap.Int(1)\noptions = {\n    \'rectangle\': bottom_rect, \'x\': ap.Int(100),\n    \'direction\': bottom_rect_direction}\nbottom_rect_timer: ap.Timer = ap.Timer(\n    on_timer, delay=ap.FPS.FPS_60,\n    options=options)\nbottom_rect_timer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_base_scale_from_point_basic_usage_x/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_base_scale_from_point_basic_usage_x/index.html" width="150" height="350"></iframe>\n\nThe `get_scale_y_from_point` and `set_scale_y_from_point` methods have the similar arguments, `scale_y` and `y`. These interfaces work the same way as the x-axis interfaces, except that the axis directions are different.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _Options(TypedDict):\n    rectangle: ap.Rectangle\n    y: ap.Int\n    direction: ap.Int\n\n\ndef on_timer(e: ap.TimerEvent, options: _Options) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    y: ap.Int = options[\'y\']\n    direction: ap.Int = options[\'direction\']\n    current_scale_y: ap.Number = rectangle.get_scale_y_from_point(y=y)\n    current_scale_y += direction * 0.03\n    rectangle.set_scale_y_from_point(scale_y=current_scale_y, y=y)\n    with ap.If(current_scale_y >= 2.0):\n        direction *= -1\n    with ap.If(current_scale_y <= 0.0):\n        direction *= -1\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\ndirection: ap.Int = ap.Int(1)\noptions: _Options = {\n    \'rectangle\': rectangle, \'y\': ap.Int(50), \'direction\': direction}\ntimer: ap.Timer = ap.Timer(\n    on_timer, delay=ap.FPS.FPS_60,\n    options=options)\ntimer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_base_scale_from_point_basic_usage_y/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_base_scale_from_point_basic_usage_y/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## get_scale_x_from_point API':
    '',

    '<!-- Docstring: apysc._display.scale_x_from_point_interface.ScaleXFromPointInterface.get_scale_x_from_point -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `get_scale_x_from_point(self, x:apysc._type.int.Int) -> apysc._type.number.Number`<hr>\n\n**[Interface summary]** Get a scale-x value from the given x-coordinate.<hr>\n\n**[Parameters]**\n\n- `x`: Int\n  - X-coordinate.\n\n<hr>\n\n**[Returns]**\n\n- `scale_x`: Number\n  - Scale-x value from the given x-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> x: ap.Int = ap.Int(100)\n>>> rectangle.set_scale_x_from_point(scale_x=ap.Number(1.5), x=x)\n>>> rectangle.get_scale_x_from_point(x=x)\nNumber(1.5)\n```':  # noqa
    '',

    '':
    '',

    '## set_scale_x_from_point API':
    '',

    '<!-- Docstring: apysc._display.scale_x_from_point_interface.ScaleXFromPointInterface.set_scale_x_from_point -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `set_scale_x_from_point(self, scale_x:apysc._type.number.Number, x:apysc._type.int.Int) -> None`<hr>\n\n**[Interface summary]** Update a scale-x value from the given x-coordinate.<hr>\n\n**[Parameters]**\n\n- `scale_x`: Number\n  - Scale-x value to set.\n- `x`: Int\n  - X-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> x: ap.Int = ap.Int(100)\n>>> rectangle.set_scale_x_from_point(scale_x=ap.Number(1.5), x=x)\n>>> rectangle.get_scale_x_from_point(x=x)\nNumber(1.5)\n```':  # noqa
    '',

    '':
    '',

    '## get_scale_y_from_point API':
    '',

    '<!-- Docstring: apysc._display.scale_y_from_point_interface.ScaleYFromPointInterface.get_scale_y_from_point -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `get_scale_y_from_point(self, y:apysc._type.int.Int) -> apysc._type.number.Number`<hr>\n\n**[Interface summary]** Get a scale-y value from the given y-coordinate.<hr>\n\n**[Parameters]**\n\n- `y`: Int\n  - Y-coordinate.\n\n<hr>\n\n**[Returns]**\n\n- `scale_y`: ap.Number\n  - Scale-y value from the given y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> y: ap.Int = ap.Int(100)\n>>> rectangle.set_scale_y_from_point(scale_y=ap.Number(1.5), y=y)\n>>> rectangle.get_scale_y_from_point(y=y)\nNumber(1.5)\n```':  # noqa
    '',

    '':
    '',

    '## set_scale_y_from_point API':
    '',

    '<!-- Docstring: apysc._display.scale_y_from_point_interface.ScaleYFromPointInterface.set_scale_y_from_point -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `set_scale_y_from_point(self, scale_y:apysc._type.number.Number, y:apysc._type.int.Int) -> None`<hr>\n\n**[Interface summary]** Update a scale-y value from the given y-coordinate.<hr>\n\n**[Parameters]**\n\n- `scale_y`: Number\n  - Scale-y value to set.\n- `y`: Int\n  - Y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> y: ap.Int = ap.Int(100)\n>>> rectangle.set_scale_y_from_point(scale_y=ap.Number(1.5), y=y)\n>>> rectangle.get_scale_y_from_point(y=y)\nNumber(1.5)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: assert_arrays_equal_and_arrays_not_equal.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# assert_arrays_equal and assert_arrays_not_equal interfaces':
    '',

    'This page explains the `assert_arrays_equal` and `assert_arrays_not_equal` function interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `assert_arrays_equal` function interface asserts that the two arrays are equal. Conversely, the `assert_arrays_not_equal` function interface asserts that the two arrays are not equal.':  # noqa
    '',

    '## See also':
    '',

    '- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)':  # noqa
    '',

    '## Basic usage':
    '',

    'Both of the `assert_arrays_equal` and `assert_arrays_not_equal` interfaces require the `left` and `right` arguments. The `msg` argument is optional.\n\nThe arguments accept a Python built-in `list` value and `Array` value.\n\nThe following example (`assert_arrays_equal` and values are equal) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(\n    left=[1, 2, 3], right=arr_1, msg=\'Values are not equal!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_arrays_equal_basic_usage_1/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_arrays_equal]\nLeft value: [1, 2, 3] right value: arr_2\n```':  # noqa
    '',

    '<iframe src="static/assert_arrays_equal_basic_usage_1/index.html" width="0" height="0"></iframe>\n\nThe following example (`assert_arrays_equal` and values are not equal) fails:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_equal(\n    left=[1, 2], right=arr_1, msg=\'Values are not equal!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_arrays_equal_basic_usage_2/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_arrays_equal]\nLeft value: [1, 2] right value: arr_2\n...\nAssertion failed: Values are not equal!\n```':  # noqa
    '',

    '<iframe src="static/assert_arrays_equal_basic_usage_2/index.html" width="0" height="0"></iframe>\n\nThe following example (`assert_arrays_not_equal` and values are not equal) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\narr_1: ap.Array = ap.Array([1, 2, 3])\nap.assert_arrays_not_equal(\n    left=[1, 2], right=arr_1, msg=\'Values are equal!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_arrays_not_equal_basic_usage_1/\')\n```':  # noqa
    '',

    '<iframe src="static/assert_arrays_not_equal_basic_usage_1/index.html" width="0" height="0"></iframe>':  # noqa
    '',

    '## Notes for the assert_equal and assert_not_equal interfaces':
    '',

    'If an `Array` value is specified to the `assert_equal` or `assert_not_equal` interface\'s values, then the `assert_arrays_equal` or `assert_arrays_not_equal` interfaces will be called instead of the `assert_equal` or `assert_not_equal` interfaces automatically.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\narr_1: ap.Array = ap.Array([1, 2, 3, 4, 5])\nap.assert_equal(\n    left=[1, 2, 3, 4, 5], right=arr_1, msg=\'Values are equal!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_arrays_equal_notes_for_the_assert_equal/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_arrays_equal]\nLeft value: [1, 2, 3, 4, 5] right value: arr_2\n```':  # noqa
    '',

    '<iframe src="static/assert_arrays_equal_notes_for_the_assert_equal/index.html" width="0" height="0"></iframe>':  # noqa
    '',

    '## assert_arrays_equal API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_arrays_equal -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_arrays_equal(left:Any, right:Any, *, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for Array values equal condition.<hr>\n\n**[Parameters]**\n\n- `left`: *\n  - Left-side value to compare.\n- `right`: *\n  - Right-side value to compare.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Notes]**\n\nThis interface is used instead of assert_equal for Array class comparison (JavaScript can not compare arrays directly, like a Python, for example, `[1, 2] === [1, 2]` becomes false).<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([1, 2, 3])\n>>> ap.assert_arrays_equal(arr_1, arr_2)\n```':  # noqa
    '',

    '':
    '',

    '## assert_arrays_not_equal API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_arrays_not_equal -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_arrays_not_equal(left:Any, right:Any, *, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for Array values not equal condition.<hr>\n\n**[Parameters]**\n\n- `left`: *\n  - Left-side value to compare.\n- `right`: *\n  - Right-side value to compare.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Notes]**\n\nThis interface is used instead of assert_not_equal for Array class comparison (JavaScript can not compare arrays directly, like a Python, for example, `[1, 2] === [1, 2]` becomes false).<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr_1: ap.Array = ap.Array([1, 2, 3])\n>>> arr_2: ap.Array = ap.Array([4, 5, 6])\n>>> ap.assert_arrays_not_equal(arr_1, arr_2)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: timer_repeat_count.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Timer class repeat_count setting':
    '',

    'This page explains the `Timer` class `repeat_count` argument setting.':
    '',

    '## What argument is this?':
    '',

    'The `repeat_count` argument setting determines the max handler calling number. For example, if you specify the 10 value, a timer calls the handler 10 times and stops.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can set the `repeat_count` parameter at the `Timer` constructor. The following example sets the timer with the 100 times `repeat_count` value when clicking the rectangle.\n\nIf the timer moves the rectangle 100 times (100-pixels to the right), the timer stops.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that a rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    options_: _RectOptions = {\'rectangle\': e.this}\n    timer: ap.Timer = ap.Timer(\n        handler=on_timer, delay=16, repeat_count=100,\n        options=options_)\n    timer.start()\n    e.this.unbind_click(handler=on_rectangle_click)\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that a timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.x += 1\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'timer_repeat_count_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_repeat_count_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## Timer constructor API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, handler:Callable[[_ForwardRef(\'timer_event.TimerEvent\'), ~_O1], NoneType], delay:Union[int, float, apysc._type.number_value_interface.NumberValueInterface, apysc._time.fps.FPS], *, repeat_count:Union[int, apysc._type.int.Int]=0, options:Union[~_O1, NoneType]=None) -> None`<hr>\n\n**[Interface summary]** Timer class to handle function calling at regular intervals.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - A handler would be called at regular intervals.\n- `delay`: Int or int or Number or float or FPS\n  - A delay between each `Handler` calling in a millisecond or FPS value. If an `FPS` value is specified, this value becomes a millisecond calculated with that FPS value (e.g., if the `FPS_60` value is specified, then `delay` becomes 16.6666667).\n- `repeat_count`: Int or int\n  - Max count of a `Handler`\'s calling. A timer stops if the `Handler`\'s calling count has reached this value. If 0 is specified, then a timer loops forever.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to pass a `Handler` callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> _ = ap.Timer(\n...     on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Timer document](https://simon-ritchie.github.io/apysc/timer.html)\n- [TimerEvent class document](https://simon-ritchie.github.io/apysc/timer_event.html)\n- [Timer class delay setting document](https://simon-ritchie.github.io/apysc/timer_delay.html)\n- [FPS enum document](https://simon-ritchie.github.io/apysc/fps.html)\n- [About the handler options\' type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## repeat_count property API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.repeat_count -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a max count value of a handler\'s calling.<hr>\n\n**[Returns]**\n\n- `repeat_count`: Int\n  - Max count of a handler\'s calling. If this value is 0, then a timer loop forever.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(\n...     on_timer, delay=33.3, repeat_count=50)\n>>> timer.repeat_count\nInt(50)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: trace.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Trace interface':
    '',

    'This page explains the `trace` function interface.':
    '',

    '## What interface is this?':
    '',

    'The `trace` function interface displays any message on the browser console. This interface behaves like the JavaScript `console.log` function.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `trace` function can accept any number of arguments and various value types.\n\nThe following example draws the rectangle. Then, the handler displays the message on the browser console when you click it(please press the F12 key).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_rectangle_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'Hello apysc!\', \'Rectangle width:\', e.this.width)\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'trace_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/trace_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## trace API':
    '',

    '<!-- Docstring: apysc._console._trace.trace -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `trace(*args:Any) -> None`<hr>\n\n**[Interface summary]** Display arguments information to console. This function saves a JavaScript `console.log` expression.<hr>\n\n**[Parameters]**\n\n- `*args`: list\n  - Any arguments to display to console.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.trace(\'Int value is:\', int_val)\n```':  # noqa
    '',

}



"""Class implementation for append line point interface.
"""

from typing import Union

from apysc._display.points_2d_interface import Points2DInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class AppendLinePointInterface(Points2DInterface):

    _points_var_name: str

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AppendLinePointInterface')
    def append_line_point(
            self, x: Union[int, Int], y: Union[int, Int]) -> None:
        """
        Append line point at the end.

        Parameters
        ----------
        x : int or Int
            X-coordinate.
        y : int or Int
            Y-coordinate.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill('#0af')
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=0, y=0),
        ...         ap.Point2D(x=0, y=50),
        ...         ap.Point2D(x=50, y=50),
        ...     ])
        >>> polygon.append_line_point(x=50, y=0)
        """
        import apysc as ap
        from apysc._type import value_util
        if not hasattr(self, '_points_var_name'):
            raise AttributeError(
                '_points_var_name attribute is not set. Please add '
                'implementation to set that value when constructor '
                'or else.')
        self.points.append(value=ap.Point2D(x=x, y=y))
        expression: str
        x_name: str = value_util.get_value_str_for_expression(value=x)
        y_name: str = value_util.get_value_str_for_expression(value=y)
        expression = (
            f'{self._points_var_name}.push([{x_name}, {y_name}]);'
            f'\n{self.variable_name}.plot({self._points_var_name});'
        )
        ap.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: assert_defined_and_undefined.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# assert_defined and assert_undefined interfaces':
    '',

    'This page explains the `assert_defined` and `assert_undefined` function interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `assert_defined` function interface asserts specified value is defined (initialized). Conversely, the `assert_undefined` function interface asserts specified value is undefined (not initialized or deleted).':  # noqa
    '',

    '## See also':
    '',

    '- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)':  # noqa
    '',

    '## Basic usage':
    '',

    'Both of the `assert_defined` and `assert_undefined` interfaces requires `value` argument and `msg` argument is optional.\n\nThe following assertion example (`assert_defined` and initialized value) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nint_val: ap.Int = ap.Int(10)\nap.assert_defined(\n    value=int_val, msg=\'Value is not defined!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_defined_basic_usage_1/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: 10\n```':  # noqa
    '',

    '<iframe src="static/assert_defined_basic_usage_1/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_defined` and the deleted value) fails:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(\n    expression=f\'{int_val.variable_name} = undefined;\')\nap.assert_defined(\n    value=int_val, msg=\'Value is not defined!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_defined_basic_usage_2/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_defined]\nRight-side variable name: i_11\nLeft value: other than undefined right value: undefined\n...\nAssertion failed: Value is not defined!\n```':  # noqa
    '',

    '<iframe src="static/assert_defined_basic_usage_2/index.html" width="0" height="0"></iframe>\n\nThe following assertion example (`assert_undefined` and the deleted value) passes:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nint_val: ap.Int = ap.Int(10)\nap.append_js_expression(\n    expression=f\'{int_val.variable_name} = undefined;\')\nap.assert_undefined(\n    value=int_val, msg=\'Value is defined!\')\n\nap.save_overall_html(\n    dest_dir_path=\'assert_undefined_basic_usage_1/\')\n```':  # noqa
    '',

    '':
    '',

    '```\n[assert_undefined]\nRight-side variable name: i_11\nLeft value: undefined right value: undefined\n```':  # noqa
    '',

    '<iframe src="static/assert_undefined_basic_usage_1/index.html" width="0" height="0"></iframe>':  # noqa
    '',

    '## assert_defined API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_defined -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_defined(value:Any, *, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for defined (not undefined) value condition.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Target value to check.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.assert_defined(int_val)\n```':  # noqa
    '',

    '':
    '',

    '## assert_undefined API':
    '',

    '<!-- Docstring: apysc._console.assertion.assert_undefined -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `assert_undefined(value:Any, *, msg:str=\'\') -> None`<hr>\n\n**[Interface summary]** JavaScript assertion interface for undefined value condition.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Target value to check.\n- `msg`: str, optional\n  - Message to display when assertion failed.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.append_js_expression(\n...     expression=f\'{int_val.variable_name} = undefined;\')\n>>> ap.assert_undefined(int_val)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: easing_enum.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Easing enum':
    '',

    'This page explains the `Easing` enum class.':
    '',

    '## What class is this?':
    '',

    'The `Easing` enum class defines each easing function. These are used at the animation interfaces, for example, the `animation_move` interface.':  # noqa
    '',

    '## Basic usage':
    '',

    'Each animation method interface, such as the `animation_move`\\, `animation_x`\\, has the `easing` argument. You can specify any of the `Easing` enums, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=50, y=50, duration=1000, easing=ap.Easing.EASE_OUT_QUINT)\n    animation_move.animation_complete(on_animation_complete_2)\n    animation_move.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=100, y=100, duration=1000, easing=ap.Easing.EASE_OUT_QUINT)\n    animation_move.animation_complete(on_animation_complete_1)\n    animation_move.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(\n    x=100, y=100, duration=1000, easing=ap.Easing.EASE_OUT_QUINT)\nanimation_move.animation_complete(on_animation_complete_1)\nanimation_move.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/easing_enum_basic_usage/index.html" width="200" height="200"></iframe>\n\nIf you skip the specification of the `easing` argument, the animation becomes a linear one:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=50, y=50, duration=1000)\n    animation_move.animation_complete(on_animation_complete_2)\n    animation_move.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_move: ap.AnimationMove = e.this.target.animation_move(\n        x=100, y=100, duration=1000)\n    animation_move.animation_complete(on_animation_complete_1)\n    animation_move.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(\n    x=100, y=100, duration=1000)\nanimation_move.animation_complete(on_animation_complete_1)\nanimation_move.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_default_setting/\')\n```':  # noqa
    '',

    '<iframe src="static/easing_enum_default_setting/index.html" width="200" height="200"></iframe>':  # noqa
    '',

    '## What difference between the ease-in, ease-out, and ease-in-out':
    '',

    '- Ease-in starts with slow speed and stops with fast speed.\n- Ease-out starts with fast speed and stops with slow speed.\n- Ease-in-out starts and stops with slow speed, and it gets faster on the way.\n\nThe following example shows the difference between the `EASE_IN_QUINT`\\, `EASE_OUT_QUINT`\\, and `EASE_IN_OUT_QUINT`\\.\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_QUINT\nEASING_2: ap.Easing = ap.Easing.EASE_OUT_QUINT\nEASING_3: ap.Easing = ap.Easing.EASE_IN_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_difference/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_difference/index.html" width="200" height="350"></iframe>':  # noqa
    '',

    '## Ease-in examples':
    '',

    'This section shows each ease-in setting result with the x-coordinate animation.\n\nThe following is the example of the `EASE_IN_SINE`, `EASE_IN_QUAD`, and `EASE_IN_CUBIC`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_SINE\nEASING_2: ap.Easing = ap.Easing.EASE_IN_QUAD\nEASING_3: ap.Easing = ap.Easing.EASE_IN_CUBIC\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_1/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_1/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_IN_QUART`, `EASE_IN_QUINT`, and `EASE_IN_EXPO`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_QUART\nEASING_2: ap.Easing = ap.Easing.EASE_IN_QUINT\nEASING_3: ap.Easing = ap.Easing.EASE_IN_EXPO\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_2/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_2/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_IN_CIRC` and `EASE_IN_BACK`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_CIRC\nEASING_2: ap.Easing = ap.Easing.EASE_IN_BACK\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_3/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_3/index.html" width="200" height="250"></iframe>\n\nThe following is the example of the `EASE_IN_ELASTIC` and `EASE_IN_BOUNCE`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_ELASTIC\nEASING_2: ap.Easing = ap.Easing.EASE_IN_BOUNCE\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_4/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_4/index.html" width="200" height="250"></iframe>':  # noqa
    '',

    '## Ease-out examples':
    '',

    'This section shows each ease-out result with the x-coordinate animation.\n\nThe following is the example of the `EASE_OUT_SINE`, `EASE_OUT_QUAD`, and `EASE_OUT_CUBIC`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_OUT_SINE\nEASING_2: ap.Easing = ap.Easing.EASE_OUT_QUAD\nEASING_3: ap.Easing = ap.Easing.EASE_OUT_CUBIC\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_out_1/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_out_1/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_OUT_QUART`, `EASE_OUT_QUINT`, and `EASE_OUT_EXPO`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_OUT_QUART\nEASING_2: ap.Easing = ap.Easing.EASE_OUT_QUINT\nEASING_3: ap.Easing = ap.Easing.EASE_OUT_EXPO\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_out_2/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_out_2/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_OUT_CIRC` and `EASE_OUT_BACK`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_OUT_CIRC\nEASING_2: ap.Easing = ap.Easing.EASE_OUT_BACK\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_out_3/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_out_3/index.html" width="200" height="250"></iframe>\n\nThe following is the example of the `EASE_OUT_ELASTIC` and `EASE_OUT_BOUNCE`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_OUT_ELASTIC\nEASING_2: ap.Easing = ap.Easing.EASE_OUT_BOUNCE\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_out_4/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_out_4/index.html" width="200" height="250"></iframe>':  # noqa
    '',

    '## Ease-in-out examples':
    '',

    'This section shows each ease-in-out setting result with the x-coordinate animation.\n\nThe following is the example of the `EASE_IN_OUT_SINE`, `EASE_IN_OUT_QUAD`, and `EASE_IN_OUT_CUBIC`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_OUT_SINE\nEASING_2: ap.Easing = ap.Easing.EASE_IN_OUT_QUAD\nEASING_3: ap.Easing = ap.Easing.EASE_IN_OUT_CUBIC\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_out_1/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_out_1/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_IN_OUT_QUART`, `EASE_IN_OUT_QUINT`, and `EASE_IN_OUT_EXPO`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_OUT_QUART\nEASING_2: ap.Easing = ap.Easing.EASE_IN_OUT_QUINT\nEASING_3: ap.Easing = ap.Easing.EASE_IN_OUT_EXPO\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=350, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_3.animation_x(\n    x=x, duration=DURATION, easing=EASING_3)\noptions = {\'easing\': EASING_3}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_out_2/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_out_2/index.html" width="200" height="350"></iframe>\n\nThe following is the example of the `EASE_IN_OUT_CIRC` and `EASE_IN_OUT_BACK`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_OUT_CIRC\nEASING_2: ap.Easing = ap.Easing.EASE_IN_OUT_BACK\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_out_3/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_out_3/index.html" width="200" height="250"></iframe>\n\nThe following is the example of the `EASE_IN_OUT_ELASTIC` and `EASE_IN_OUT_BOUNCE`:\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _EasingOptions(TypedDict):\n    easing: ap.Easing\n\n\nDURATION: int = 1000\nEASING_1: ap.Easing = ap.Easing.EASE_IN_OUT_ELASTIC\nEASING_2: ap.Easing = ap.Easing.EASE_IN_OUT_BOUNCE\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=50, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(on_animation_complete_2, options=options)\n    animation_x.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle],\n        options: _EasingOptions) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    animation_x: ap.AnimationX = e.this.target.animation_x(\n        x=100, duration=DURATION, easing=options[\'easing\'])\n    animation_x.animation_complete(\n        on_animation_complete_1, options=options)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=250, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=150, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=250, width=50, height=50)\n\nx: int = 100\nanimation_x: ap.AnimationX = rectangle_1.animation_x(\n    x=x, duration=DURATION, easing=EASING_1)\noptions: _EasingOptions = {\'easing\': EASING_1}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nanimation_x = rectangle_2.animation_x(\n    x=x, duration=DURATION, easing=EASING_2)\noptions = {\'easing\': EASING_2}\nanimation_x.animation_complete(on_animation_complete_1, options=options)\nanimation_x.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./easing_enum_ease_in_out_4/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/easing_enum_ease_in_out_4/index.html" width="200" height="250"></iframe>':  # noqa
    '',

}



"""Class implementation for the animation_reset interface.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_interface import VariableNameInterface


class AnimationResetInterface(VariableNameInterface):

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationResetInterface')
    def animation_reset(self) -> None:
        """
        Stop all animations and reset.

        References
        ----------
        - animation_reset interface document
            - https://simon-ritchie.github.io/apysc/animation_reset.html

        Examples
        --------
        >>> from typing_extensions import TypedDict
        >>> import apysc as ap
        >>> class RectOptions(TypedDict):
        ...     rectangle: ap.Rectangle
        >>> def on_timer(
        ...         e: ap.TimerEvent,
        ...         options: RectOptions) -> None:
        ...     rectangle: ap.Rectangle = options['rectangle']
        ...     rectangle.animation_reset()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.animation_x(
        ...     x=100,
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        >>> options: RectOptions = {'rectangle': rectangle}
        >>> ap.Timer(on_timer, delay=750, options=options).start()
        """
        import apysc as ap
        expression: str = (
            f'{self.variable_name}.timeline().stop();'
        )
        ap.append_js_expression(expression=expression)



"""Class implementation for the line alpha value.
"""

from typing import Generic
from typing import TypeVar
from typing import Union

from apysc._animation.animation_base import AnimationBase
from apysc._animation.easing import Easing
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_interface import VariableNameInterface

_T = TypeVar('_T', bound=VariableNameInterface)


class AnimationLineAlpha(AnimationBase[_T], Generic[_T]):
    """
    The animation class for the line alpha.

    References
    ----------
    - animation_line_alpha interface document
        - https://simon-ritchie.github.io/apysc/animation_line_alpha.html
    - Animation interfaces duration setting document
        - https://simon-ritchie.github.io/apysc/animation_duration.html
    - Animation interfaces delay setting document
        - https://simon-ritchie.github.io/apysc/animation_delay.html
    - Each animation interface return value document
        - https://simon-ritchie.github.io/apysc/animation_return_value.html  # noqa
    - Sequential animation setting document
        - https://simon-ritchie.github.io/apysc/sequential_animation.html
    - animation_parallel interface document
        - https://simon-ritchie.github.io/apysc/animation_parallel.html
    - Easing enum document
        - https://simon-ritchie.github.io/apysc/easing_enum.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> sprite.graphics.line_style(
    ...     color='#fff', thickness=5, alpha=1.0)
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> animation: ap.AnimationLineAlpha = rectangle.animation_line_alpha(
    ...     alpha=0.0,
    ...     duration=1500,
    ...     easing=ap.Easing.EASE_OUT_QUINT,
    ... )
    >>> _ = animation.start()
    """

    _line_alpha: Number

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='AnimationLineAlpha')
    def __init__(
            self,
            *,
            target: _T,
            alpha: Union[float, Number],
            duration: Union[int, Int] = 3000,
            delay: Union[int, Int] = 0,
            easing: Easing = Easing.LINEAR) -> None:
        """
        The animation class for the line alpha.

        Parameters
        ----------
        target : VariableNameInterface
            A target instance of the animation target
            (e.g., `Rectangle` instance).
        alpha : float or Number
            The final line alpha of the animation.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        variable_name: str = expression_variables_util.\
            get_next_variable_name(
                type_name=var_names.ANIMATION_LINE_ALPHA)
        self._line_alpha = to_apysc_val_from_builtin.\
            get_copied_number_from_builtin_val(float_or_num=alpha)
        self._set_basic_animation_settings(
            target=target,
            duration=duration,
            delay=delay,
            easing=easing)
        super(AnimationLineAlpha, self).__init__(
            variable_name=variable_name)

    def _get_animation_func_expression(self) -> str:
        """
        Get a animation function expression.

        Returns
        -------
        expression : str
            Animation function expression.
        """
        from apysc._type import value_util
        line_alpha_str: str = value_util.get_value_str_for_expression(
            value=self._line_alpha)
        return f'\n  .attr({{"stroke-opacity": {line_alpha_str}}});'

    def _get_complete_event_in_handler_head_expression(self) -> str:
        """
        Get an expression to be inserted into the complete event
        handler's head.

        Returns
        -------
        expression : str
            An expression to insert into the complete event
            handler's head.
        """
        from apysc._display.line_alpha_interface import LineAlphaInterface
        expression: str = ''
        if isinstance(self._target, LineAlphaInterface):
            self._target._initialize_line_alpha_if_not_initialized()
            expression = (
                f'{self._target._line_alpha.variable_name} = '
                f'{self._line_alpha.variable_name};'
            )
        return expression



"""Elif (else if) branch instruction implementations.
"""

from typing import Any
from typing import Dict
from typing import Optional

from apysc._branch.if_base import IfBase
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean


class Elif(IfBase):
    """
    A class to append the `else if` branch instruction expression.

    References
    ----------
    - Elif document
        - https://simon-ritchie.github.io/apysc/elif.html
    - Each branch instruction class's scope variables reverting setting
        - https://simon-ritchie.github.io/apysc/branch_instruction_variables_reverting_setting.html  # noqa

    Notes
    -----
    - Currently the apysc can not initialize condition value
        in the constructor. For example, the following raises
        an exception:
    - You can only use this class immediately after the
        `If` or `Elif` statement.

    Examples
    --------
    >>> import apysc as ap
    >>> any_value: ap.Int = ap.Int(10)
    >>> condition_1: ap.Boolean = any_value >= 10
    >>> condition_2: ap.Boolean = any_value >= 5
    >>> with ap.If(condition_1):
    ...     # Do something here
    ...     pass
    >>> with ap.Elif(condition_2):
    ...     # Do something else here
    ...     pass
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Elif')
    def __init__(
            self,
            condition: Optional[Boolean],
            *,
            locals_: Optional[Dict[str, Any]] = None,
            globals_: Optional[Dict[str, Any]] = None) -> None:
        """
        A class to append the `else if` branch instruction expression.

        Notes
        -----
        - Currently the apysc can not initialize condition value
            in the constructor. For example, the following raises
            exception:
        - You can only use this class immediately after the
            `If` or `Elif` statement.

        ```py
        import apysc as ap
        with ap.Elif(any_value >= 10):
            ...
        ```

        You can avoid this exception by predefining condition
        value, as follows:

        >>> import apysc as ap
        >>> any_value: ap.Int = ap.Int(10)
        >>> condition_1: ap.Boolean = any_value >= 10
        >>> condition_2: ap.Boolean = any_value >= 5
        >>> with ap.If(condition_1):
        ...     # Do something here
        ...     pass
        >>> with ap.Elif(condition_2):
        ...     # Do something else here
        ...     pass

        Parameters
        ----------
        condition : Boolean or None
            Boolean value to be used for judgment.
        locals_ : dict or None, default None
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameInterface
            variables (like Int, Sprite) at the end of an `Elif`
            scope. This setting is useful when you don't want to
            update each variable by implementing the `Elif` scope.
        globals_ : dict or None, default None
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.

        References
        ----------
        - Elif document
            - https://simon-ritchie.github.io/apysc/elif.html
        - Each branch instruction class's scope variables reverting setting
            - https://simon-ritchie.github.io/apysc/branch_instruction_variables_reverting_setting.html  # noqa
        """
        super(Elif, self).__init__(
            condition=condition, locals_=locals_, globals_=globals_)

    def _append_enter_expression(self) -> None:
        """
        Append else if branch instruction starting expression.

        Raises
        ------
        ValueError
            If the last scope is not If or Elif.
        """
        import apysc as ap
        if not self._last_scope_is_if_or_elif():
            raise ValueError(
                'Elif interface can only use right after If or Elif '
                'interfaces.'
                '\n\nMaybe you are using Int or String, or anything else'
                ' comparison expression at Elif constructor (e.g., '
                '`with Elif(any_value == 10, ...):`).'
                '\nCurrently that specifying expression directly is not '
                'supported so please'
                ' define condition seperately as follows:'
                '\ncondition: Boolean = any_value == 10'
                '\n...'
                '\nwith Elif(condition, ....):')
        if self._condition is None:
            raise ValueError(
                'Elif expression\'s condition argument can\'t be set None.')
        expression: str = (
            f'else if ({self._condition.variable_name}) {{'
        )
        ap.append_js_expression(expression=expression)

    def _set_last_scope(self) -> None:
        """
        Set expression last scope value.
        """
        from apysc._expression import last_scope
        from apysc._expression.last_scope import LastScope
        last_scope.set_last_scope(value=LastScope.ELIF)



"""This module is for the SVG `line to` (L) path data
class implementation.
"""

from typing import Any
from typing import Union

from apysc._geom.path_data_base import PathDataBase
from apysc._geom.path_x_interface import PathXInterface
from apysc._geom.path_y_interface import PathYInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.string import String


class PathLineTo(PathDataBase, PathXInterface, PathYInterface):
    """
    Path data class for the SVG `line to` (L).

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color='#fff', thickness=3)
    >>> path: ap.Path = sprite.graphics.draw_path(
    ...     path_data_list=[
    ...         ap.PathMoveTo(x=0, y=50),
    ...         ap.PathLineTo(x=50, y=50),
    ...     ])
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathLineTo')
    def __init__(
            self, x: Union[int, Int], y: Union[int, Int], *,
            relative: Union[bool, Boolean] = False) -> None:
        """
        Path data class for the SVG `line to` (L).

        Parameters
        ----------
        x : Int or int
            X-coordinate of the destination point.
        y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=3)
        >>> path: ap.Path = sprite.graphics.draw_path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=50),
        ...         ap.PathLineTo(x=50, y=50),
        ...     ])
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        from apysc._geom.path_label import PathLabel
        super(PathLineTo, self).__init__(
            path_label=PathLabel.LINE_TO,
            relative=relative)
        self.x = get_copied_int_from_builtin_val(integer=x)
        self.y = get_copied_int_from_builtin_val(integer=y)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathLineTo')
    def _get_svg_str(self) -> str:
        """
        Get an SVG path string created with the current setting.

        Returns
        -------
        svg_str : str
            An SVG path string was created with the
            current setting.
        """
        from apysc._type import value_util
        svg_char: String = self._get_svg_char()
        svg_char_str: str = value_util.get_value_str_for_expression(
            value=svg_char)
        x_str: str = value_util.get_value_str_for_expression(value=self._x)
        y_str: str = value_util.get_value_str_for_expression(value=self._y)
        svg_str: str = (
            f'{svg_char_str} + String({x_str}) + " " '
            f'+ String({y_str})'
        )
        return svg_str

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathLineTo')
    def update_path_data(
            self, x: Union[int, Int], y: Union[int, Int],
            *,
            relative: Union[bool, Boolean] = False) -> None:
        """
        Update the path data settings.

        Parameters
        ----------
        x : Int or int
            X-coordinate of the destination point.
        y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).

        Examples
        --------
        >>> import apysc as ap
        >>> line_to: ap.PathLineTo = ap.PathLineTo(x=50, y=50)
        >>> line_to.update_path_data(x=100, y=150)
        >>> line_to.x
        Int(100)

        >>> line_to.y
        Int(150)
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_boolean_from_builtin_val
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        self.x = get_copied_int_from_builtin_val(integer=x)
        self.y = get_copied_int_from_builtin_val(integer=y)
        self.relative = get_copied_boolean_from_builtin_val(
            bool_val=relative)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathLineTo')
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap
        if not isinstance(other, PathLineTo):
            result: ap.Boolean = ap.Boolean(False)
            return result
        return (
            self.x == other.x
            and self.y == other.y
            and self.relative == other.relative)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathLineTo')
    def __ne__(self, other: Any) -> Any:
        """
        Not equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap
        result: ap.Boolean = self == other
        result = result.not_
        return result



"""Class implementation for line style-related interface.

See Also
--------
- graphics_clear_interface
"""

from typing import Dict
from typing import Optional
from typing import TypeVar
from typing import Union

from apysc._display.line_caps import LineCaps
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_joints import LineJoints
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.revert_interface import RevertInterface
from apysc._type.string import String

StrOrString = TypeVar('StrOrString', str, String)


class LineStyleInterface(RevertInterface):

    _line_color: String
    _line_thickness: Int
    _line_alpha: Number
    _line_cap: String
    _line_joints: String
    _line_dot_setting: Optional[LineDotSetting]
    _line_dash_setting: Optional[LineDashSetting]
    _line_round_dot_setting: Optional[LineRoundDotSetting]
    _line_dash_dot_setting: Optional[LineDashDotSetting]

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_style(
            self, color: StrOrString,
            *,
            thickness: Union[int, Int] = 1,
            alpha: Union[float, Number] = 1.0,
            cap: Optional[LineCaps] = None,
            joints: Optional[LineJoints] = None,
            dot_setting: Optional[LineDotSetting] = None,
            dash_setting: Optional[LineDashSetting] = None,
            round_dot_setting: Optional[LineRoundDotSetting] = None,
            dash_dot_setting: Optional[LineDashDotSetting] = None) -> None:
        """
        Set line style values.

        Parameters
        ----------
        color : String or str
            Hexadecimal color string. e.g., '#00aaff'
        thickness : Int or int, default 1
            Line thickness (minimum value is 1).
        alpha : float or Number, default 1.0
            Line color opacity (0.0 to 1.0).
        cap : LineCaps or None, default None
            Line cap (edge style) setting. The not line-related
            graphics (e.g., Rectangle ignores this, conversely
            used by Polyline) ignore this setting.
        joints : LineJoints or None, default None
            Line vertices (joints) style setting. The not
            polyline-related graphics (e.g., Rectangle ignores
            this, conversely used by Polyline) ignore this setting.
        dot_setting : LineDotSetting or None, default None
            Dash setting. If this is specified, it makes a line dashed.
        dash_setting : LineDashSetting or None, default None
            Dash setting. If this is specified, it makes a line dashed.
        round_dot_setting : LineRoundDotSetting or None, default None
            Round dot setting. If this is specified, it makes a line
            round dotted. Notes: since this style uses a cap setting,
            it overrides cap and line thickness settings. And it
            increases the amount of line size.
            If you want to adjust to the same width of a normal line
            when using move_to and line_to interfaces,
            add half-round size to start x-coordinate and subtract
            from end e-coordinate.
            e.g.,
            `this.move_to(x + round_size / 2, y)`,
            `this.line_to(x - round_size / 2, y)`
        dash_dot_setting : LineDashDotSetting or None, default None
            Dash dot (1-dot chain) setting. If this is specified,
            it makes a line 1-dot chained.

        References
        ----------
        - Graphics line_style interface document
            - https://simon-ritchie.github.io/apysc/graphics_line_style.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5, alpha=0.5,
        ...     cap=ap.LineCaps.ROUND)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50)
        >>> line.line_color
        String('#ffffff')

        >>> line.line_thickness
        Int(5)

        >>> line.line_alpha
        Number(0.5)

        >>> line.line_cap
        String('round')
        """
        import apysc as ap
        from apysc._color import color_util
        from apysc._validation import color_validation
        from apysc._validation import display_validation
        from apysc._validation import number_validation

        self._initialize_line_color_if_not_initialized()
        self._initialize_line_thickness_if_not_initialized()
        self._initialize_line_alpha_if_not_initialized()

        if color != '':
            color = color_util.complement_hex_color(
                hex_color_code=color)
        self._line_color.value = color
        number_validation.validate_integer(integer=thickness)
        number_validation.validate_num_is_gt_zero(num=thickness)
        self._line_thickness = ap.Int(thickness)
        number_validation.validate_num(num=alpha)
        if isinstance(alpha, ap.Number):
            alpha_: ap.Number = alpha
        else:
            alpha_ = ap.Number(alpha)
        color_validation.validate_alpha_range(alpha=alpha)
        self._line_alpha = alpha_
        self._set_line_cap(cap=cap)
        self._set_line_joints(joints=joints)
        self._line_dot_setting = dot_setting
        self._line_dash_setting = dash_setting
        self._line_round_dot_setting = round_dot_setting
        self._line_dash_dot_setting = dash_dot_setting
        display_validation.validate_multiple_line_settings_isnt_set(
            any_instance=self)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def _set_line_joints(self, *, joints: Optional[LineJoints]) -> None:
        """
        Set line joints setting to attribute.

        Parameters
        ----------
        joints : LineJoints or None, default None
            Line vertices (joints) style setting.
        """
        import apysc as ap
        from apysc._validation.display_validation import validate_line_joints
        if joints is None:
            joints = LineJoints.MITER
        validate_line_joints(joints=joints)
        self._line_joints = ap.String(joints.value)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def _set_line_cap(self, *, cap: Optional[LineCaps]) -> None:
        """
        Set line cap setting to attribute.

        Parameters
        ----------
        cap : LineCaps or None, default None
            Line cap (edge style) setting.
        """
        import apysc as ap
        from apysc._validation.display_validation import validate_line_cap
        if cap is None:
            cap = LineCaps.BUTT
        validate_line_cap(cap=cap)
        self._line_cap = ap.String(cap.value)

    def _initialize_line_color_if_not_initialized(self) -> None:
        """
        Initialize _line_color attribute if this interface does
        not initialize it yet.
        """
        if hasattr(self, '_line_color'):
            return
        self._line_color = String('')

    def _initialize_line_thickness_if_not_initialized(self) -> None:
        """
        Initialize _line_thickness attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, '_line_thickness'):
            return
        self._line_thickness = Int(1)

    def _initialize_line_alpha_if_not_initialized(self) -> None:
        """
        Initialize _line_alpha attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_line_alpha'):
            return
        self._line_alpha = Number(1.0)

    def _initialize_line_cap_if_not_initialized(self) -> None:
        """
        Initialize _line_cap attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_line_cap'):
            return
        self._line_cap = String(LineCaps.BUTT.value)

    def _initialize_line_joints_if_not_initialized(self) -> None:
        """
        Initialize _line_joints attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_line_joints'):
            return
        self._line_joints = String(LineJoints.MITER.value)

    def _initialize_line_dot_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_dot_setting attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, '_line_dot_setting'):
            return
        self._line_dot_setting = None

    def _initialize_line_dash_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_dash_setting attribute if this
        interface does not initialize it yet.
        """
        if hasattr(self, '_line_dash_setting'):
            return
        self._line_dash_setting = None

    def _initialize_line_round_dot_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_round_dot_setting attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, '_line_round_dot_setting'):
            return
        self._line_round_dot_setting = None

    def _initialize_line_dash_dot_setting_if_not_initialized(self) -> None:
        """
        Initialize _line_dash_dot_setting attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, '_line_dash_dot_setting'):
            return
        self._line_dash_dot_setting = None

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_color(self) -> String:
        """
        Get current line color.

        Returns
        -------
        line_color : String
            Current line color (hexadecimal string, e.g., '#00aaff').
            If not be set, this interface returns a blank string.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5, alpha=0.5,
        ...     cap=ap.LineCaps.ROUND)
        >>> sprite.graphics.line_color
        String('#ffffff')
        """
        from apysc._type import value_util
        self._initialize_line_color_if_not_initialized()
        return value_util.get_copy(value=self._line_color)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_thickness(self) -> Int:
        """
        Get current line thickness.

        Returns
        -------
        line_thickness : Int
            Current line thickness.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5, alpha=0.5)
        >>> sprite.graphics.line_thickness
        Int(5)
        """
        from apysc._type import value_util
        self._initialize_line_thickness_if_not_initialized()
        return value_util.get_copy(value=self._line_thickness)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_alpha(self) -> Number:
        """
        Get current line color opacity.

        Returns
        -------
        line_alpha : Number
            Current line opacity (0.0 to 1.0).

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5, alpha=0.5,
        ...     cap=ap.LineCaps.ROUND)
        >>> sprite.graphics.line_alpha
        Number(0.5)
        """
        from apysc._type import value_util
        self._initialize_line_alpha_if_not_initialized()
        return value_util.get_copy(value=self._line_alpha)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_cap(self) -> String:
        """
        Get current line cap (edge) style setting.

        Returns
        -------
        line_cap : String
            Current line cap (edge) style setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5, alpha=0.5,
        ...     cap=ap.LineCaps.ROUND)
        >>> sprite.graphics.line_cap
        String('round')
        """
        self._initialize_line_cap_if_not_initialized()
        return self._line_cap

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_joints(self) -> String:
        """
        Get current line joints (vertices) style setting.

        Returns
        -------
        line_joints : String
            Current line joints (vertices) style setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5,
        ...     joints=ap.LineJoints.ROUND)
        >>> sprite.graphics.line_joints
        String('round')
        """
        self._initialize_line_joints_if_not_initialized()
        return self._line_joints

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_dot_setting(self) -> Optional[LineDotSetting]:
        """
        Get current line dot setting.

        Returns
        -------
        line_dot_setting : LineDotSetting or None
            Current line dot setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5,
        ...     dot_setting=ap.LineDotSetting(dot_size=5))
        >>> sprite.graphics.line_dot_setting.dot_size
        Int(5)
        """
        self._initialize_line_dot_setting_if_not_initialized()
        return self._line_dot_setting

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineStyleInterface')
    def line_dash_setting(self) -> Optional[LineDashSetting]:
        """
        Get current line dash setting.

        Returns
        -------
        line_dash_setting : LineDashSetting or None
            Current line dash setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5,
        ...     dash_setting=ap.LineDashSetting(
        ...         dash_size=10, space_size=5))
        >>> sprite.graphics.line_dash_setting.dash_size
        Int(10)

        >>> sprite.graphics.line_dash_setting.space_size
        Int(5)
        """
        self._initialize_line_dash_setting_if_not_initialized()
        return self._line_dash_setting

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineRoundDotSetting')
    def line_round_dot_setting(self) -> Optional[LineRoundDotSetting]:
        """
        Get current line round dot setting.

        Returns
        -------
        line_round_dot_setting : LineRoundDotSetting or None
            Current line round dot setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5,
        ...     round_dot_setting=ap.LineRoundDotSetting(
        ...         round_size=6, space_size=3))
        >>> sprite.graphics.line_round_dot_setting.round_size
        Int(6)

        >>> sprite.graphics.line_round_dot_setting.space_size
        Int(3)
        """
        self._initialize_line_round_dot_setting_if_not_initialized()
        return self._line_round_dot_setting

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='LineRoundDotSetting')
    def line_dash_dot_setting(self) -> Optional[LineDashDotSetting]:
        """
        Get current line dash dot setting.

        Returns
        -------
        line_dash_dot_setting : LineDashDotSetting or None
            Current line dash dot setting.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(
        ...     color='#fff', thickness=5,
        ...     dash_dot_setting=ap.LineDashDotSetting(
        ...         dot_size=2, dash_size=5, space_size=3))
        >>> sprite.graphics.line_dash_dot_setting.dot_size
        Int(2)

        >>> sprite.graphics.line_dash_dot_setting.dash_size
        Int(5)

        >>> sprite.graphics.line_dash_dot_setting.space_size
        Int(3)
        """
        self._initialize_line_dash_dot_setting_if_not_initialized()
        return self._line_dash_dot_setting

    _line_color_snapshots: Dict[str, str]
    _line_thickness_snapshots: Dict[str, int]
    _line_alpha_snapshots: Dict[str, float]
    _line_cap_snapshots: Dict[str, str]
    _line_joints_snapshots: Dict[str, str]
    _line_dot_setting_snapshots: Dict[str, Optional[LineDotSetting]]
    _line_dash_setting_snapshots: Dict[str, Optional[LineDashSetting]]
    _line_round_dot_setting_snapshots: Dict[
        str, Optional[LineRoundDotSetting]]
    _line_dash_dot_setting_snapshots: Dict[str, Optional[LineDashDotSetting]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_color_if_not_initialized()
        self._initialize_line_thickness_if_not_initialized()
        self._initialize_line_alpha_if_not_initialized()
        self._initialize_line_cap_if_not_initialized()
        self._initialize_line_joints_if_not_initialized()
        self._initialize_line_dot_setting_if_not_initialized()
        self._initialize_line_dash_setting_if_not_initialized()
        self._initialize_line_round_dot_setting_if_not_initialized()
        self._initialize_line_dash_dot_setting_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_color_snapshots',
            value=self._line_color._value, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_thickness_snapshots',
            value=int(self._line_thickness._value),
            snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_alpha_snapshots',
            value=self._line_alpha._value, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_cap_snapshots',
            value=self._line_cap._value, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_joints_snapshots',
            value=self._line_joints._value, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_dot_setting_snapshots',
            value=self._line_dot_setting, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_dash_setting_snapshots',
            value=self._line_dash_setting, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_round_dot_setting_snapshots',
            value=self._line_round_dot_setting, snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name='_line_dash_dot_setting_snapshots',
            value=self._line_dash_dot_setting, snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._line_color._value = self._line_color_snapshots[snapshot_name]
        self._line_thickness._value = self._line_thickness_snapshots[
            snapshot_name]
        self._line_alpha._value = self._line_alpha_snapshots[snapshot_name]
        self._line_cap._value = self._line_cap_snapshots[snapshot_name]
        self._line_joints._value = self._line_joints_snapshots[snapshot_name]

        self._line_dot_setting = self._line_dot_setting_snapshots[
            snapshot_name]
        self._line_dash_setting = self._line_dash_setting_snapshots[
            snapshot_name]
        self._line_round_dot_setting = self._line_round_dot_setting_snapshots[
            snapshot_name]
        self._line_dash_dot_setting = self._line_dash_dot_setting_snapshots[
            snapshot_name]



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_rotation_around_center.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# GraphicsBase rotation_around_center interface':
    '',

    'This page explains the `GraphicsBase` class (base class of each graphic, such as the `Rectangle`) `rotation_around_center` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `rotation_around_center` property interface can set the rotation angle to its instance (rotation value around its center point).':  # noqa
    '',

    '## Basic usage':
    '',

    'The `rotation_around_center` interface accepts the `int` or `Int` value.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set the cyan fill color and draw the rectangle.\nsprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\ncyan_rect: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\ncyan_rect.rotation_around_center = ap.Int(30)\n\n# Set the magenta fill color and draw the rectangle.\nsprite.graphics.begin_fill(color=\'#f0a\', alpha=0.5)\nmagenta_rect: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n# Append the rotation angle with the incremental addition (the result\n# rotation will be 60 degrees).\nmagenta_rect.rotation_around_center += ap.Int(30)\nmagenta_rect.rotation_around_center += ap.Int(30)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_base_rotation_around_center_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_base_rotation_around_center_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Notes':
    '',

    'This interface supports only the graphics instances currently. The container instances, such as the `Sprite` instance, are not supported (due to the HTML (SVG) specification).':  # noqa
    '',

    '## rotation_around_center property API':
    '',

    '<!-- Docstring: apysc._display.rotation_around_center_interface.RotationAroundCenterInterface.rotation_around_center -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a rotation value around the center of this instance.<hr>\n\n**[Returns]**\n\n- `rotation_around_center`: Int\n  - Rotation value around the center of this instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.rotation_around_center = ap.Int(45)\n>>> rectangle.rotation_around_center\nInt(45)\n```':  # noqa
    '',

}



"""This module is for SVG's `continual 3D bezier curve` (S)
path data class implementations.
"""

from typing import Any
from typing import Union

from apysc._geom.path_control_x_interface import PathControlXInterface
from apysc._geom.path_control_y_interface import PathControlYInterface
from apysc._geom.path_data_base import PathDataBase
from apysc._geom.path_dest_x_interface import PathDestXInterface
from apysc._geom.path_dest_y_interface import PathDestYInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.boolean import Boolean
from apysc._type.int import Int
from apysc._type.string import String


class PathBezier3DContinual(
        PathDataBase, PathControlXInterface, PathControlYInterface,
        PathDestXInterface, PathDestYInterface):
    """
    Path data class for SVG's `continual 3D bezier curve` (S).

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color='#fff', thickness=3)
    >>> path: ap.Path = sprite.graphics.draw_path(
    ...     path_data_list=[
    ...         ap.PathMoveTo(x=0, y=50),
    ...         ap.PathBezier3D(
    ...             control_x1=0, control_y1=0,
    ...             control_x2=50, control_y2=0,
    ...             dest_x=50, dest_y=50),
    ...         ap.PathBezier3DContinual(
    ...             control_x=100, control_y=100,
    ...             dest_x=100, dest_y=50),
    ...     ])
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathBezier3DContinual')
    def __init__(
            self,
            control_x: Union[int, Int],
            control_y: Union[int, Int],
            dest_x: Union[int, Int],
            dest_y: Union[int, Int],
            *,
            relative: Union[bool, Boolean] = False) -> None:
        """
        Path data class for SVG's `continual 3D bezier curve` (S).

        Parameters
        ----------
        control_x : Int or int
            X-coordinate of the bezier's control point.
        control_y : Int or int
            Y-coordinate of the bezier's control point.
        dest_x : Int or int
            X-coordinate of the destination point.
        dest_y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path coordinates
            are relative or not (absolute).

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=3)
        >>> path: ap.Path = sprite.graphics.draw_path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=50),
        ...         ap.PathBezier3D(
        ...             control_x1=0, control_y1=0,
        ...             control_x2=50, control_y2=0,
        ...             dest_x=50, dest_y=50),
        ...         ap.PathBezier3DContinual(
        ...             control_x=100, control_y=100,
        ...             dest_x=100, dest_y=50),
        ...     ])
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        from apysc._geom.path_label import PathLabel
        super(PathBezier3DContinual, self).__init__(
            path_label=PathLabel.BEZIER_3D_CONTINUAL,
            relative=relative)
        self.control_x = get_copied_int_from_builtin_val(
            integer=control_x)
        self.control_y = get_copied_int_from_builtin_val(
            integer=control_y)
        self.dest_x = get_copied_int_from_builtin_val(integer=dest_x)
        self.dest_y = get_copied_int_from_builtin_val(integer=dest_y)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathBezier3DContinual')
    def _get_svg_str(self) -> str:
        """
        Get a path's SVG string created with the current setting.

        Returns
        -------
        svg_str : str
            An SVG path string was created with the
            current setting.
        """
        from apysc._type.value_util import get_value_str_for_expression
        svg_char: String = self._get_svg_char()
        svg_char_str: str = get_value_str_for_expression(
            value=svg_char)
        control_x_str: str = get_value_str_for_expression(
            value=self._control_x)
        control_y_str: str = get_value_str_for_expression(
            value=self._control_y)
        dest_x_str: str = get_value_str_for_expression(value=self._dest_x)
        dest_y_str: str = get_value_str_for_expression(value=self._dest_y)
        svg_str: str = (
            f'{svg_char_str} + String({control_x_str}) '
            f'+ " " + String({control_y_str}) '
            f'+ " " + String({dest_x_str}) '
            f'+ " " + String({dest_y_str})'
        )
        return svg_str

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathBezier3DContinual')
    def update_path_data(
            self,
            control_x: Union[int, Int],
            control_y: Union[int, Int],
            dest_x: Union[int, Int],
            dest_y: Union[int, Int],
            *,
            relative: Union[bool, Boolean] = False) -> None:
        """
        Update the path's data settings.

        Parameters
        ----------
        control_x : Int or int
            X-coordinate of the bezier's control point.
        control_y : Int or int
            Y-coordinate of the bezier's control point.
        dest_x : Int or int
            X-coordinate of the destination point.
        dest_y : Int or int
            Y-coordinate of the destination point.
        relative : bool or Boolean, default False
            A boolean value indicates whether the path
            coordinates are relative or not (absolute).

        Examples
        --------
        >>> import apysc as ap
        >>> bezier_3d_continual = ap.PathBezier3DContinual(
        ...     control_x=100, control_y=100,
        ...     dest_x=100, dest_y=50)
        >>> bezier_3d_continual.update_path_data(
        ...     control_x=150, control_y=150,
        ...     dest_x=150, dest_y=100)
        >>> bezier_3d_continual.control_x
        Int(150)

        >>> bezier_3d_continual.control_y
        Int(150)

        >>> bezier_3d_continual.dest_x
        Int(150)

        >>> bezier_3d_continual.dest_y
        Int(100)
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_boolean_from_builtin_val
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        self.control_x = get_copied_int_from_builtin_val(
            integer=control_x)
        self.control_y = get_copied_int_from_builtin_val(
            integer=control_y)
        self.dest_x = get_copied_int_from_builtin_val(integer=dest_x)
        self.dest_y = get_copied_int_from_builtin_val(integer=dest_y)
        self.relative = get_copied_boolean_from_builtin_val(
            bool_val=relative)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathBezier3DContinual')
    def __eq__(self, other: Any) -> Any:
        """
        Equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap
        if not isinstance(other, PathBezier3DContinual):
            result: ap.Boolean = ap.Boolean(False)
            return result
        return (
            self.control_x == other.control_x
            and self.control_y == other.control_y
            and self.dest_x == other.dest_x
            and self.dest_y == other.dest_y
            and self.relative == other.relative)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='PathBezier3DContinual')
    def __ne__(self, other: Any) -> Any:
        """
        Not equal comparison method.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap
        result: ap.Boolean = self == other
        result = result.not_
        return result



"""This module is for the translation mapping data of the
following document:

Document file: branch_instruction_variables_reverting_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Each branch instruction class\'s scope variable reverting setting':
    '',

    'This page explains each branch instruction class (like the `If`\\, `Elif`\\, and `Else`) scope variables reverting setting.':  # noqa
    '',

    '## These interfaces execute With statement code':
    '',

    'These interfaces execute the code in each branch instruction regardless of the condition, and variables are updated.\n\nFor example, the following code of the condition is `False`\\, but the value of int is 20 on the Python:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition):\n    int_1 += 10\nassert int_1 == 20\n```':  # noqa
    '',

    'This condition is skipped in JavaScript (converted code) since the condition is not satisfied.':  # noqa
    '',

    '## Scope variables reverting setting':
    '',

    'The `If`, `Elif`, and `Else` classes have the arguments of the `locals_` and `globals_` (basically set the `locals()` and `globals` built-in functions return value). If these arguments are specified, the scope variables are reverted when ended each scope (e.g., `If` scope).\n\nThis interface is occasionally helpful when you don\'t want to update the variables in each branch instruction scope.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(False)\nint_1: ap.Int = ap.Int(10)\n\nwith ap.If(condition, locals_=locals(), globals_=globals()):\n    int_1 += 10\nassert int_1 == 10\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [If class](if.md)\n- [Elif class](elif.md)\n- [Else class](else.md)':
    '',

}
